#include <QFileDialog>
#include <QApplication>
#include <QDesktopWidget>
#include "prefgeneral.h"
#include "prefgeneralwidget.h"
#include "mplayer/informations.h"
#include "ui_prefgeneralwidget.h"

namespace Preferences {

GeneralWidget::GeneralWidget(const General &general, QWidget *parent)
: QWidget(parent), ui(new Ui::Ui_GeneralWidget), m_general(new General) {
	*m_general = general;
	ui->setupUi(this);
	
	connect(ui->find_mplayer_button, SIGNAL(clicked()), this, SLOT(findMPlayer()));
	connect(ui->expand_resolution_button, SIGNAL(clicked()), this, SLOT(setExpandWithResolution()));
	
	static MPlayer::Informations *info = MPlayer::Informations::get();
	
	ui->mplayer_path_edit->setText(m_general->mplayerPath);
	ui->auto_add_combo->setCurrentIndex(m_general->autoAddFiles);
	ui->pause_check->setChecked(m_general->pauseWhenMinimized);
	ui->play_restored_check->setChecked(m_general->playWhenRestored);
	ui->additional_options_edit->setText(m_general->additionalOptions);
	
	int index = -1;
	ui->vo_combo->addItems(info->videoOutputs());
	if ((index = info->videoOutputs().indexOf(m_general->videoOutput)) == -1)
		ui->vo_combo->setEditText(m_general->videoOutput);
	else
		ui->vo_combo->setCurrentIndex(index);
	ui->software_equalizer_check->setChecked(m_general->useSoftwareEqualizer);
	ui->expand_group->setChecked(m_general->useExpand);
	ui->expand_width_edit->setValidator(new QDoubleValidator(this));
	ui->expand_width_edit->setText(QString::number(m_general->expandWidth));
	ui->expand_height_edit->setValidator(new QDoubleValidator(this));
	ui->expand_height_edit->setText(QString::number(m_general->expandHeight));
	ui->expand_fullscreen_check->setChecked(m_general->expandOnlyFullScreen);
		
	ui->ao_combo->addItems(info->audioOutputs());
	if ((index = info->audioOutputs().indexOf(m_general->audioOutput)) == -1)
		ui->ao_combo->setEditText(m_general->audioOutput);
	else
		ui->ao_combo->setCurrentIndex(index);
	ui->software_volume_check->setChecked(m_general->useSoftwareVolume);
	ui->amplification_spin->setValue(m_general->volumeAmplification);
	ui->reset_volume_check->setChecked(m_general->resetVolume);
	ui->volume_slider->setValue(m_general->defaultVolume);
}

GeneralWidget::~GeneralWidget() {
	delete ui;
	delete m_general;
}

void GeneralWidget::findMPlayer() {
	QString path = QFileDialog::getOpenFileName(this);
	if (!path.isEmpty())
		ui->mplayer_path_edit->setText(path);
}

const General &GeneralWidget::general() const {
	m_general->mplayerPath = ui->mplayer_path_edit->text().trimmed();
	m_general->autoAddFiles = General::AutoAddFiles(ui->auto_add_combo->currentIndex());
	m_general->pauseWhenMinimized = ui->pause_check->isChecked();
	m_general->playWhenRestored = ui->play_restored_check->isChecked();
	m_general->additionalOptions = ui->additional_options_edit->text().trimmed();
	
	m_general->videoOutput = ui->vo_combo->currentText().trimmed();
	m_general->useSoftwareEqualizer = ui->software_equalizer_check->isChecked();
	m_general->useExpand = ui->expand_group->isChecked();
	m_general->expandWidth = ui->expand_width_edit->text().toDouble();
	m_general->expandHeight = ui->expand_height_edit->text().toDouble();
	m_general->expandOnlyFullScreen = ui->expand_fullscreen_check->isChecked();
	
	m_general->audioOutput = ui->ao_combo->currentText().trimmed();
	m_general->useSoftwareVolume = ui->software_volume_check->isChecked();
	m_general->volumeAmplification = ui->amplification_spin->value();
	m_general->resetVolume = ui->reset_volume_check->isChecked();
	m_general->defaultVolume = ui->volume_slider->value();
	return *m_general;
}

void GeneralWidget::setExpandWithResolution() {
	static QSize size = QApplication::desktop()->size();
	ui->expand_width_edit->setText(QString::number(size.width()));
	ui->expand_height_edit->setText(QString::number(size.height()));
}

}
