#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include "mplayer.h"

class RecentStack;

class MainWindow : public QMainWindow {
	Q_OBJECT
public:
	MainWindow(QWidget *parent = 0);
	~MainWindow();
public slots:
	void setFullScreen(bool full);
protected:
	virtual bool eventFilter(QObject *obj, QEvent *event);
	virtual void closeEvent(QCloseEvent *event);
	virtual void mouseMoveEvent(QMouseEvent *event);
	virtual void mouseDoubleClickEvent(QMouseEvent *event);
	virtual void mousePressEvent(QMouseEvent *event);
	virtual void dragEnterEvent(QDragEnterEvent *event);
	virtual void dropEvent(QDropEvent *event);
	virtual void wheelEvent(QWheelEvent *event);
	virtual void hideEvent(QHideEvent *event);
	virtual void showEvent(QShowEvent *event);
private slots:
	void open();
	void openRecent();
	void changeAspectRatio(QAction *act);
	void changeStaysOnTop(QAction *act);
	void slotStateChanged(MPlayer::State state);
	void slotResized();
	void setVideoSize(double rate);
	void changeVideoSize(QAction *act);
	void changeCurrentSubtitles(QAction *act);
	void updateSubtitles(const QStringList &files);
	void updateCurrentSubtitleIndexes(const QList<int> &indexes);
	void showPreferencesDialog();
	void clearSubtitleList();
	void addSubtitles();
	void showEqualizer();
	void crop(QAction *act);
	void updateSyncDelay(int msec);
	void increaseVolume();
	void decreaseVolume();
	void forward();
	void forwardMore();
	void forwardMuchMore();
	void backward();
	void backwardMore();
	void backwardMuchMore();
	void stepDownSubtitle();
	void stepUpSubtitle();
	void increaseSyncDelay();
	void decreaseSyncDelay();
	void togglePlayPause();
	void updateRecentActions(const RecentStack &files);
	void updateRecentSize(int size);
	void clearRecentFiles();
	void showMouseMenu(const QPoint &pos);
	void showAboutDialog();
	void selectABSection();
	void showABRepeatDialog();
	void togglePlayListVisibility();
	void adjustSizeForDock(bool visible);
	void updatePlayText();
	void increaseSpeed();
	void decreaseSpeed();
	void restoreSpeed();
	void doubleSpeed();
	void halfSpeed();
	void updateSpeed(double speed);
	void updateFinishedFile();
	void updateStopped(qint64 time);
private:
	class Data;
	friend class Data;
	Data *d;
};

#endif
